


#r "nuget: Plotly.NET,2.0.0-alpha2"
#r "nuget: BioFSharp,2.0.0-beta4"
#r "nuget: BioFSharp.BioDB,2.0.0-beta4"
#r "nuget: FSharp.Stats,0.3.0-beta"
#r "nuget: FSharpAux,1.0.0"
#r "netstandard.dll"

open System
open Plotly.NET
open Plotly.NET.Axis
open BioFSharp
open FSharp.Stats
open StyleParam
open FSharpAux
open Fitting.LinearRegression.OrdinaryLeastSquares

#time

FSharp.Stats.ServiceLocator.setEnvironmentPathVariable @"C:\Users\bvenn\source\repos\FSharp.Stats\lib"
FSharp.Stats.Algebra.LinearAlgebra.Service()

let myAxis name = LinearAxis.init(Title=name,Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true)
let myAxisWODes name = LinearAxis.init(Title=name,Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true,Tickfont=Font.init(StyleParam.FontFamily.Arial,Size=6.))
let myAxisRange name range = LinearAxis.init(Title=name,Range=StyleParam.Range.MinMax(range), Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true)
let myAxisRangeWODes name range = LinearAxis.init(Title=name,Range=StyleParam.Range.MinMax(range), Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true,Tickfont=Font.init(StyleParam.FontFamily.Arial,Size=6.))
let myAxisLog name = LinearAxis.init(AxisType=AxisType.Log,Title=name, Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true)
let styleChart x y chart = chart |> Chart.withX_Axis (myAxis x) |> Chart.withY_Axis (myAxis y)
let styleChartwR x y (min,max) chart = chart |> Chart.withX_Axis (myAxisRange x (min,max)) |> Chart.withY_Axis (myAxis y)


let getReg (vec:Vector<float>) rep = vec.[0..rep]
Vector.init 2 (fun _ -> 2.)

let fst3 (a,b,c) = a
let snd3 (a,b,c) = b
let trd3 (a,b,c) = c

let rnd = System.Random()


module Annotation = 
    
    open FSharpAux
    
    type ChlamyMapping =
        {
        JGI43full           :string
        JGI55full           :string
        JGI43               :string
        JGI55               :string
        MapManGMM           :string []
        MapManDescription   :string []
        GOTerm              :string []
        GODescription       :string []
        Synonym             :string
        SubLocTerm          :string
        SubLoc              :string
        }
    let createChlamyMapping (str:string) = 
        let tmp = str.Split([|'\t'|])
        {
        JGI43full           = tmp.[0]
        JGI55full           = tmp.[1]
        JGI43               = tmp.[2]
        JGI55               = tmp.[3]
        MapManGMM           = tmp.[4].Split([|'|'|])
        MapManDescription   = tmp.[5].Split([|'|'|])
        GOTerm              = tmp.[6].Split([|'|'|])
        GODescription       = tmp.[7].Split([|'|'|])
        Synonym             = tmp.[8]
        SubLocTerm          = tmp.[9]
        SubLoc              = tmp.[10]
        }
    
    //Translate JGI4.3.1 to JGI5.5 and contains further annotations for each cre number
    let readJGIMapping = 
        System.IO.File.ReadAllLines(@"\\fp0.uni-kl.de\DFS\tu\bio\biotech\Organisation\AG Muehlhaus\Annotation\Chlamy\chlamyMapping_jgi431_jgi55.txt")
        |> Array.tail
        |> Array.map (fun x -> 
            createChlamyMapping x
            )
    
    
    ///Map to translate MapManGmm-Term to MapManDescription for most of MapManBins
    let gmm2Des =
        readJGIMapping
        |> Array.map (fun x -> 
            let zip = Array.zip x.MapManGMM x.MapManDescription
            zip
            |> Array.map (fun (gmm,des) -> 
                let tmp1 = gmm.Split('.') |> Array.scanReduce (fun acc elem -> acc + "." + elem)
                let tmp2 = des.Split('.') |> Array.scanReduce (fun acc elem -> acc + "." + elem)
                Array.zip tmp1 tmp2
                    )
            )
        |> Array.concat
        |> Array.concat
        |> Array.distinct
        |> Map.ofArray
    
    let gmmDesToJGI55 str = 
        readJGIMapping
        |> Array.filter (fun x -> 
            x.MapManDescription 
            |> Array.exists (fun des -> des.StartsWith str) 
            )
        |> Array.map (fun x -> x.JGI55)


    //Map to translate MapManGmm-Term to MapManDescription for most of MapManBins
    let jgi55ToInfo =
        readJGIMapping
        |> Array.map (fun x -> 
            x.JGI55,(sprintf "%s\t%s\t%s\t%s\t%s\t%s" x.JGI55 (x.MapManGMM|> String.concat ";") (x.MapManDescription|> String.concat ";") (x.GODescription|> String.concat ";") x.SubLoc x.Synonym)
            )
        |> Map.ofArray

    //Map to translate MapManGmm-Term to MapManDescription for most of MapManBins
    let jgi55ToDes =
        readJGIMapping
        |> Array.map (fun x -> 
            x.JGI55,(x.MapManDescription|> String.concat ";").Replace("\"",""))
        |> Map.ofArray

    let jgi55ToDesStr (str :string)=
        let str = (str.Split '.').[0..1] |> String.concat "."
        if jgi55ToDes.ContainsKey str then jgi55ToDes.[str] else ""

    //Map to translate MapManGmm-Term to MapManDescription for most of MapManBins
    let jgi55ToGODes =
        readJGIMapping
        |> Array.map (fun x -> 
            x.JGI55,(x.GODescription|> String.concat ";").Replace("\"",""))
        |> Map.ofArray

    let jgi55ToGODesStr (str :string)=
        let truncatedStr = (str.Split '.').[0..1] |> String.concat "."
        if jgi55ToGODes.ContainsKey truncatedStr then jgi55ToGODes.[truncatedStr] else ""

    //Map to translate MapManGmm-Term to MapManDescription for most of MapManBins
    let jgi55ToSyn =
        readJGIMapping
        |> Array.map (fun x -> 
            x.JGI55,x.Synonym.Replace("\"",""))
        |> Map.ofArray

    let jgi55ToSynStr (str :string) =
        let truncatedStr = (str.Split '.').[0..1] |> String.concat "."
        //if jgi55ToSyn.ContainsKey truncatedStr then 
        //    jgi55ToSyn.[truncatedStr] 
        //else 
        let bioDBQuery = BioDB.FaToolDb.Queries.getSynonyms truncatedStr
        if bioDBQuery = [] then 
            "" 
        else bioDBQuery |> List.map snd |> String.concat ";"

    //Map to translate MapManGmm-Term to MapManDescription for most of MapManBins
    let jgi55ToLoc =
        readJGIMapping
        |> Array.map (fun x -> 
            x.JGI55,x.SubLoc
            )
        |> Map.ofArray

    let jgi55ToLocStr (str:string) =
        let str' = (str.Split '.').[0..1] |> String.concat "."
        if jgi55ToLoc.ContainsKey str' then jgi55ToLoc.[str'] else ""

module DTW =
                           
    type warpingPaths = {
        Distance: float
        Paths   : Matrix<float>
        }
        
    //window: see   : Only allow for shifts up to this amount away from the two diagonals.
    //maxDist: see  : Stop if the returned distance measure will be larger than this value.
    //maxStep: see  : Do not allow steps larger than this value.
    //maxLengthDiff : Return infinity if difference in length of two series is larger.
    //penalty       : Penalty to add if compression or expansion is applied (on top of the distance).
    //psi           : Relaxation to ignore begin and/or end of sequences (for cylical sequencies)
    ///Dynamic Time Warping.
    ///The full matrix of all warping paths is build.
    ///x: First sequence
    ///y: Second sequence
    ///returns: (DTW distance, DTW matrix)
    let warp window maxDist maxStep maxLengthDiff penalty psi (x:array<float>) (y:array<float>) = 
        let r,c = x.Length, y.Length
        match maxLengthDiff with
        | Some x when abs(r-c) > x -> None 
        | _  -> 
        let window = 
            match window with
            | Some x -> x
            | None   -> max r c             
        let maxStep = 
            match maxStep with
            | Some x -> Some (x**2.)
            | None   -> None 
        let maxDist = 
            match maxDist with
            | Some x -> Some (x**2.)
            | None   -> None
        let penalty = 
            match penalty with
            | Some x -> (x**2.)
            | None   -> 0. 
        let psi = 
            match psi with
            | Some x -> x
            | None   -> 0    
        let dtw = 
            let tmp = Matrix.create (r+1) (c+1) infinity 
            for i = 0 to psi do 
                tmp.[0,i] <- 0.
                tmp.[i,0] <- 0.
            tmp
        ///
        let rec innerLoop upper prevLastUnderMaxDist lastUnderMaxDis i0 i1 i j = 
            if j = upper then 
                i+1, lastUnderMaxDis
            else 
                /// euclidean distance is used
                let d = (x.[i] - y.[j])**2.
                match maxStep with 
                | Some x when d > x -> 
                    innerLoop upper prevLastUnderMaxDist lastUnderMaxDis i0 i1 i (j+1)
                | _ ->                   
                    /// cost function
                    dtw.[i1,j+1] <- d + (List.min [ dtw.[i0, j];dtw.[i0, j + 1] + penalty;dtw.[i1, j] + penalty]) 
                    match maxDist with 
                    | Some x when dtw.[i1, j + 1] <= x ->
                        innerLoop upper prevLastUnderMaxDist (j) i0 i1 i (j+1)
                    | Some x -> 
                        dtw.[i1, j + 1] <- infinity 
                        if prevLastUnderMaxDist < (j+1) then 
                            (i+1, lastUnderMaxDis)
                        else 
                        innerLoop upper prevLastUnderMaxDist lastUnderMaxDis i0 i1 i (j+1)
                    | None  -> 
                        innerLoop upper prevLastUnderMaxDist lastUnderMaxDis i0 i1 i (j+1)
        ///
        let rec outerLoop lastUnderMaxDist i = 
            if i = r then Some (dtw,i)
            else 
                let prevLastUnderMaxDist = 
                    if lastUnderMaxDist = -1 then
                        infinity |> int |> (*) -1
                    else
                        lastUnderMaxDist
                let last_under_max_dist, i0, i1 = -1, i, i+1
                let lower = 
                    let tmp = i - (max 0 (r - c)) - window + 1
                    max 0 tmp
                let upper = 
                    let tmp = i + (max 0 (c - r)) + window
                    min c tmp
                let nextIdx,prevLastUnderMaxDist = innerLoop upper prevLastUnderMaxDist lastUnderMaxDist i0 i1 i lower
                match maxDist with 
                | Some x when lastUnderMaxDist = -1 ->
                    None 
                | Some x -> 
                    outerLoop lastUnderMaxDist (i+1)
                | None ->
                    outerLoop lastUnderMaxDist (i+1)
        let (dtw') = outerLoop 0 0
        match dtw' with 
        | None -> 
            Some ({Distance=infinity;Paths=dtw}) 
        | Some (dtw',i1) ->
            Matrix.inplace_mapi (fun m n x -> sqrt x) dtw' |> ignore
            let d = 
                //if psi = 0 then 
                        //take the cumulated error in the furthers corner (when square, that last cell)
                        dtw.[i1, (min c (c + window - 1))]  
            //    else 
            //        let ir = i1
            //        let ic = min (c + window - 1)
            //        vr = dtw.get[ir-psi:ir+1, ic]
            //        vc = dtw[ir, ic-psi:ic+1]
            //        mir = np.argmin(vr)
            //        mic = np.argmin(vc)
            //        if vr[mir] < vc[mic]:
            //            dtw[ir-psi+mir+1:ir+1, ic] = -1
            //            d = vr[mir]
            //        else:
            //            dtw[ir, ic - psi + mic + 1:ic+1] = -1
            //            d = vc[mic]
            Some ({Distance=d;Paths=dtw}) 
        
    open FSharpAux

//DTW.warp None None None None None None [|0.1;4.1;0.1;0.1;0.1;|] [|0.1;0.1;0.1;6.1;0.1;|]        
//DTW.warp None None None None None None [|1.;2.;4.;2.;5.;3.;2.;3.;2.;5.;|] [|1.;1.;2.;4.;3.;5.;3.;2.;3.;2.;|]        
      
        
    /// Returns the best warping path.
    let bestPath (paths:Matrix<float>) =
        let m,n = 
            let m',n' = paths.Dimensions
            m'-1,n'-1
        let pInit = if paths.[m,n] <> -1. then [m-1,n-1] else []
        let rec loop m n path =
            if m <= 0 || n <= 0 then
                match path with 
                | h::t -> t
                | []   -> []
            else 
            let pos = [|paths.[m - 1, n - 1]; paths.[m - 1, n]; paths.[m, n - 1]|]    
            let c = Array.foldi (fun i acc x -> if x < pos.[acc] then i else acc ) 0 pos
            let m',n' = 
                if c = 0 then
                    (m-1),(n-1)
                elif c = 1 then 
                    (m-1),(n)
                else
                    (m),(n-1)
            if paths.[m',n'] <> -1. then 
                loop m' n' ((m'-1,n'-1)::path)
            else loop m' n' (path)
        loop m n pInit        
        
    ///
    let warping_Path window maxDist maxStep maxLengthDiff penalty psi (x:array<float>) (y:array<float>) = 
        let res = warp window maxDist maxStep maxLengthDiff penalty psi x y
        match res with 
        | Some x -> bestPath x.Paths
        | None -> failwith "Warping not possible."
            
    ///
    let distance window maxDist maxStep maxLengthDiff penalty psi (x:array<float>) (y:array<float>) = 
        let res = warp window maxDist maxStep maxLengthDiff penalty psi x y
        match res with 
        | Some x -> x.Distance
        | None -> failwith "Warping not possible."

    let distanceDefault (x:seq<float>) (y:seq<float>) =
        distance None None None None None None (x|> Array.ofSeq) (y|> Array.ofSeq)
           
    /// Aligns source to target. Given a sourceX it returns the mapping of the xVal closest to
    /// sourceX to its targetX counterPart after alignment.
    let align (target:(float*float)[]) (source:(float*float)[]) =
        let xt,yt = target |> Array.unzip
        let xs,ys = source |> Array.unzip
        let optimalPath = warping_Path None None None None None None yt ys
        List.map (fun (ytIdx,ysIdx) -> xt.[ytIdx],ys.[ysIdx]) optimalPath 
    
    /// Aligns source to target. Given a sourceX it returns the mapping of the xVal closest to
    /// sourceX to its targetX counterPart after alignment.
    let align' (target:(float*float)[]) (source:(float*float)[]) sourceX =
        let xt,yt = target |> Array.unzip
        let xs,ys = source |> Array.unzip
        let optimalPath = warping_Path None None None None None None yt ys
        List.map (fun (ytIdx,ysIdx) -> xs.[ysIdx],xt.[ytIdx]) optimalPath 
        |> List.minBy (fun (xs,xt) -> abs(xs-sourceX))
        
    let s1 = 
        [|0.;1.;2.;3.;2.;1.;0.;0.;0.;0.;0.;0.;0.;0.;0.;0.;0.;0.;1.;2.;1.;0.|] 
        |> Array.mapi (fun i x -> float i,x)
        
    let s2 = 
        [|0.;0.;0.;0.;0.;1.;2.;2.5;2.;1.;0.;0.;0.;0.;0.;0.;0.;0.;0.;0.;0.1;0.2;0.1;0.;0.;0.5;1.;0.5;0.|] 
        |> Array.mapi (fun i x -> float i + 20.,x)
        
    let s2' = 
        align s1 s2
    
        
    let zNorm (y:float []) = 
        let m = Seq.mean y
        let s = Seq.stDev y 
        y
        |> Array.map (fun x -> (x-m)/s)
        
        

type Protein = {
    ID      : string
    MajorID : string
    MolWeight: float
    //Qvalue : float
    //Score : float
    Intensities: float []
    IBaq : float []
    LFQ : float []
    } with static member create id majID weight inten ibaq lfq=
            {
            ID = id
            MajorID = majID 
            MolWeight = weight
            Intensities = inten
            IBaq = ibaq
            LFQ = lfq
            }

type IntMeasurement = 
    | Intensity
    | LFQ
    | IBAQ

let getIntensity intensityType transform (x:Protein) = 
    match intensityType with
    | IBAQ      -> Array.map transform x.IBaq
    | Intensity -> Array.map transform x.Intensities
    | LFQ       -> Array.map transform x.LFQ

let readFrame foldername =
    System.IO.File.ReadAllLines(@"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\data\Fred\CPLXome LPA2\20180420 MS " + foldername + " txt\proteinGroups.txt")
    |> Array.tail
    |> Array.map (fun x -> 
        let tmp = x.Split '\t'
        let id = tmp.[0]
        let majid = tmp.[1]
        let molWeight = tmp.[121]       |> float
        let qValue = tmp.[124]
        let score = tmp.[125]
        let intensity = tmp.[163..198]  |> Array.map float
        let iBAQ = tmp.[200..235]       |> Array.map float
        let LFQ = tmp.[236..271]        |> Array.map float
        //String.concat "\t" [id;majid;molWeight;qValue;score;String.concat "\t" intensity;String.concat "\t"iBAQ;String.concat "\t"LFQ]
        Protein.create id majid molWeight intensity iBAQ LFQ
        )
    |> Array.filter (fun x -> 
        not (x.ID.Contains "CON") && not (x.ID.Contains "REV" )
        //&& (x.ID |> String.filter (fun s -> s = 'C') |> fun c -> c.Length = 1)
        )



let wt1  = readFrame "JBbnAida001" 
let wt2  = readFrame "JBbnBida001" 
let wt3  = readFrame "JBbnCida001" 
let mut1 = readFrame "JBbnDida001" 
let mut2 = readFrame "JBbnEida001" 
let mut3 = readFrame "JBbnFida001"  

let datasets = [|wt1;wt2;wt3;mut1;mut2;mut3|]
let colors= [|"F25930";"FABB01";"FFEA48";"48BCD7";"0081C7";"0B3D6F";|]
let laneHeader = [|"wt1";"wt2";"wt3";"mut1";"mut2";"mut3"|]










module Normalization =
    

    /// defines proteins existing in every lane and are unique in protein group
    let intersUnique = 
        [
        wt1  |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        wt2  |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        wt3  |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        mut1 |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        mut2 |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        mut3 |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        ]
        |> Set.intersectMany


    let plotLaneIntensities (transform:float -> float) name (datasets:Protein[] []) intType = 
        datasets
        |> Array.map (fun lane ->
            lane
            //|> Array.filter (fun protein -> inters.Contains protein.ID)
            |> Array.sumBy (fun t -> Array.sum (Array.map transform (getIntensity intType transform t)))
            )
        |> Array.mapi (fun i x -> 
            //printfn "%f" x
            laneHeader.[i],x)
        //|> Chart.Combine
        |> Chart.Line
        |> styleChart "lane" "intensity sum"
        |> Chart.withTitle name
        |> Chart.Show

    
    //plotLaneIntensities (fun si -> if si = 0. then 0. else log si)  "intersect proteins, log lfq sum"  [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.LFQ 
    plotLaneIntensities id                                          "all proteins,     lfq sum"   [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.LFQ
    plotLaneIntensities id                                          "all proteins,     IBAq sum"  [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.IBAQ
    plotLaneIntensities id                                          "all proteins,     intensity sum"  [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.Intensity
    
    let plotSliceIntensities (transform :float -> float) name aggregator (datasets:Protein[] []) intType =
        datasets
        |> Array.map (fun x -> x (*|> Array.filter (fun x -> inters.Contains x.ID)*) |> Array.sortBy (fun x -> x.ID) |> Array.map (fun x -> Array.map transform (getIntensity intType transform x)))
        |> Array.mapi (fun i x -> 
            x 
            |> JaggedArray.transpose
            |> Array.map aggregator
            |> Array.indexed
            |> Chart.Line
            |> Chart.withTraceName laneHeader.[i])
        |> Chart.Combine
        |> styleChart "slice" "intensity sum"
        |> Chart.withTitle name
        |> Chart.withSize (1100.,750.)
        |> Chart.Show

    //plotSliceIntensities (fun si -> if si = 0. then 0. else log si)  "All proteins, log intensities sum"   Array.sum [|wt1;wt2;wt3;mut1;mut2;mut3|]
    //plotSliceIntensities id      "All proteins,     intensities sum"   Array.sum [|wt1;wt2;wt3;mut1;mut2;mut3|]

    let correctedValues intType transform = 
        // summed lane intensities
        let summedlanes =
            [|wt1;wt2;wt3;mut1;mut2;mut3|]
            |> Array.map (fun lane ->
                lane
                //|> Array.filter (fun protein -> inters.Contains protein.ID)
                |> Array.sumBy (fun t -> Array.sum (getIntensity intType transform t))
                )
        let laneSumsMean = Seq.mean summedlanes
        let laneSumsCorrection = 
            summedlanes 
            |> Array.map (fun x -> x/laneSumsMean)

        let corrected = 
            [|wt1;wt2;wt3;mut1;mut2;mut3|]
            |> Array.mapi (fun laneIndex lane -> 
                lane 
                |> Array.map (fun protein ->
                    let corrected = 
                        (getIntensity intType transform protein)
                        |> Array.mapi (fun sliceIndex slice -> 
                            slice/laneSumsCorrection.[laneIndex])
                    match intType with
                    | IntMeasurement.LFQ ->         {protein with LFQ = corrected;  Intensities = [||];         IBaq = [||]}
                    | IntMeasurement.Intensity ->   {protein with LFQ = [||];       Intensities = corrected;    IBaq = [||]}
                    | IntMeasurement.IBAQ ->        {protein with LFQ = [||];       Intensities = [||];         IBaq = corrected}
                    )
                )
        corrected
    
let correctedLFQs =         Normalization.correctedValues IntMeasurement.LFQ            id
let correctedIBAQss =       Normalization.correctedValues IntMeasurement.IBAQ           id
let correctedIntensities =  Normalization.correctedValues IntMeasurement.Intensity      id

let writeDataFrame (input:Protein[][]) outputPath =
    input
    |> Array.mapi (fun j sample -> 
        sample
        //|> Array.take 5
        |> Array.filter (fun x -> not (x.ID.Contains "iRT"))
        |> Array.mapi (fun i x ->   
            if i%50=0 then printfn "%i_%i" j i
            //let truncID = 
            //    x.ID.Split '.'
            //    |> fun s -> 
            //        if s.Length > 4 then printfn "Warning %s" x.ID
            //        s.[0..1] |> String.concat "."
            //let syn = Annotation.jgi55ToSynStr truncID
            //let (mapMan,mapManDes) = 
            //    (BioFSharp.BioDB.FaToolDb.Queries.getOntologyterms BioDB.FaToolDb.MapManOntology truncID).OntologyGroups |> List.unzip |> fun (a,b) -> (a |> String.concat ";").Replace("GMM:",""),b |> String.concat ";" 
            //let annotationGO = (BioFSharp.BioDB.FaToolDb.Queries.getOntologyterms BioDB.FaToolDb.GeneOntology truncID).OntologyGroups |> List.map snd |> String.concat ";" 
            //let annotationLoc = (BioFSharp.BioDB.FaToolDb.Queries.getOntologyterms BioDB.FaToolDb.SubCellularLocalisationOntology truncID).OntologyGroups |> List.map snd |> String.concat ";" 
            //sprintf "%s\t%s\t%f\t%s\t%s\t%s\t%s\t%s" x.ID syn x.MolWeight mapMan mapManDes annotationGO annotationLoc (x.Intensities |> Array.map string |> String.concat "\t")
            sprintf "%s\t%f\t%s" x.ID x.MolWeight (x.Intensities |> Array.map string |> String.concat "\t")
            

            )
        )
    |> Array.mapi (fun i sample -> 
        let intensityHeader = Array.init 36 (fun x -> sprintf "Intensity_%02i" (x+1)) |> String.concat "\t"
        //let header = sprintf "Identifier\tTrivialName\tMolWeight\tMapMan\tMapManDescription\tGO\tLocalization\t%s" intensityHeader
        let header = sprintf "Identifier\tMolWeight\t%s" intensityHeader
        let rows = Array.append [|header|] sample
        System.IO.File.WriteAllLines(outputPath + laneHeader.[i] + ".txt",rows)
        )

writeDataFrame datasets @"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\results\forMichael_v9\intensitiesRaw_"
writeDataFrame correctedIntensities @"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\results\forMichael_v9\intensitiesNormed_"

correctedIntensities
|> Array.map (fun x -> x |> Array.map (fun x -> x.Intensities))
|> Array.concat
|> Array.concat
|> Array.filter (fun x -> x <> 0.)
|> Array.map log10
|> Chart.Histogram
|> styleChart "log10(intensity)" "count"
|> Chart.Show

Colors.Table.Office.lightOrange

let plotRelativeIntensity cre=
    let getWtMeanIntensity =
        datasets.[0..2] |> Array.concat |> Array.filter (fun x -> not (x.ID.Contains ";"))
        |> Array.filter (fun x -> x.ID.StartsWith cre)
        |> Array.map (fun x -> x.Intensities |> Array.sum)
        |> Seq.mean
        |> fun x -> Shape.init(ShapeType.Line,log10 x,log10 x,0.,350.,Line=Line.init(Color="ED7D31"))

    let getMuMeanIntensity =
        datasets.[3..5] |> Array.concat |> Array.filter (fun x -> not (x.ID.Contains ";"))
        |> Array.filter (fun x -> x.ID.StartsWith cre)
        |> Array.map (fun x -> x.Intensities |> Array.sum)
        |> Seq.mean
        |> fun x -> Shape.init(ShapeType.Line,log10 x,log10 x,0.,350.,Line=Line.init(Color="5B9BD5"))

    datasets
    |> Array.map (fun x -> x |> Array.map (fun x -> x.Intensities |> Array.sum) )
    |> Array.concat
    |> Array.filter (fun x -> x <> 0.)
    |> Array.map log10
    |> Chart.Histogram
    |> styleChart "log10(protein intensity)" "count"
    |> Chart.withShapes [getWtMeanIntensity;getMuMeanIntensity]
    |> Chart.withTitle cre
    |> Chart.Show
    
plotRelativeIntensity "Cre01.g042200"
plotRelativeIntensity "Cre03.g151200"
plotRelativeIntensity "Cre06.g278245"
plotRelativeIntensity "Cre06.g272450"
plotRelativeIntensity "Cre02.g105650"
plotRelativeIntensity "Cre-1.g2717030"
plotRelativeIntensity "Cre02.g073850"

Normalization.plotLaneIntensities id "lane ibaq sums (after intensity sum correction)" correctedIBAQss            IntMeasurement.IBAQ
Normalization.plotSliceIntensities id "slice ibaq sums (after intensity sum correction)" Array.sum correctedIBAQss  IntMeasurement.IBAQ

Normalization.plotLaneIntensities id "lane intensity sums (after intensity sum correction)" correctedIntensities            IntMeasurement.Intensity
Normalization.plotSliceIntensities id "slice intensity sums (after intensity sum correction)" Array.sum correctedIntensities  IntMeasurement.Intensity




///////////////////////////
/////////////////////////// single warping
///////////////////////////
///plot bestPaths and warping
module Warping =
    correctedIntensities
    |> Array.mapi (fun laneIndex lane -> 
        lane 
        |> Array.map (fun protein -> 
            laneIndex,(protein.MolWeight,protein.ID),protein.LFQ
            )
        )
    |> Array.concat
    |> Array.groupBy (snd3 >> snd)
    |> Array.filter (fun (identifier,items) -> 
        not (identifier.Contains ";") &&
        items.Length = 6 &&
        not (identifier.StartsWith "CON")
        )
    |> Array.map (fun (i,x) -> Array.map trd3 x)
    |> Array.item 1
    |> fun z ->
        let median = 
            z |> JaggedArray.transpose |> Array.map Seq.median
            |> Array.mapi (fun i x -> float (i+1),x)

        z
        |> Array.map (fun lane -> 
            let laneSeq = lane |> Array.mapi (fun i x -> float (i+1),x)
            let maxSliceIndex = laneSeq |> Array.maxBy snd |> fst
            let xt,yt = median |> Array.unzip
            let xs,ys = laneSeq |> Array.unzip
            let optimalPath = DTW.warping_Path None None None None None None yt ys
            let newtrace =  List.map (fun (ytIdx,ysIdx) -> xt.[ytIdx],ys.[ysIdx]) optimalPath 
            let maxSliceIndex_new = newtrace |> Seq.maxBy snd |> fst

            newtrace
            |> fun x -> Chart.Line(x,"warped",Color=colors.[5])
            |> fun x -> 
                Chart.Combine [Chart.Line(laneSeq,"signal",Color=colors.[3]);Chart.Line(median,"mean",Color=colors.[1]);x],
                optimalPath
            )
        |> Array.unzip
        |> fun (a,b) -> 
            a
            |> Chart.Stack 2
            |> Chart.withSize(1000.,800.)
            |> Chart.Show
            b
            |> Array.map (Chart.Line)
            |> Chart.Combine
            |> Chart.Show



    ///plot WARPING boxplots
    correctedIntensities 
    |> Array.mapi (fun laneIndex lane -> 
        lane 
        |> Array.map (fun protein -> 
            {|LaneIndex = laneIndex; Weight = protein.MolWeight;ID= protein.ID;LFQ=protein.LFQ|}
            )
        )
    |> Array.concat
    |> Array.groupBy (fun x -> x.ID)
    |> Array.filter (fun (identifier,items) -> 
        not (identifier.Contains ";") &&
        items.Length = 6 &&
        not (identifier.StartsWith "CON")
        )
    |> Array.mapi (fun proteinIndex (identifier,x) -> proteinIndex,x |> Array.map (fun z -> z.LFQ))
    //|> Array.item 1
    |> Array.map (fun (proteinIndex,z) ->
        //median trace of this protein
        let median = 
            z 
            |> JaggedArray.transpose 
            |> Array.map Seq.median
            |> Array.mapi (fun i x -> float (i+1),x)

        z
        |> Array.mapi (fun laneIndex trace -> 
            let traceSeq = trace |> Array.mapi (fun i x -> float (i+1),x)
            let maxSliceIndex = traceSeq |> Array.maxBy snd |> fst
            let xt,yt = median |> Array.unzip
            let xs,ys = traceSeq |> Array.unzip
            let optimalPath = DTW.warping_Path None None None None None None yt ys
            let newtrace =  List.map (fun (ytIdx,ysIdx) -> xt.[ytIdx],ys.[ysIdx]) optimalPath 
            let maxSliceIndex_new = newtrace |> Seq.maxBy snd |> fst

            let peakOffset = maxSliceIndex - maxSliceIndex_new
            {|Lane=laneIndex;MaxIntSlice=maxSliceIndex;PeakOffset=peakOffset|}
            )
        )
    |> Array.concat
    |> Array.groupBy (fun x -> x.Lane)
    |> fun x -> 
        printfn "%i" x.Length
        x
    |> Array.map (fun (laneIndex,items) ->
        //everything for one lane
        items
        |> Array.groupBy (fun x -> x.MaxIntSlice)
        |> Array.map (fun (sliceIndex,d) -> 
            //everything for one slice within one lane
            d 
            |> Array.map (fun x -> x.PeakOffset)
            |> fun s -> Chart.BoxPlot(y=s,Name = string sliceIndex,Color=colors.[laneIndex%6])
            )
        |> Chart.Combine
        )
    |> Chart.Combine
    |> styleChart "sliceIndex" "offset to warped (maxLFQSliceIndex - warpedSliceIndex"
    |> Chart.withSize (1000.,900.)
    |> Chart.Show

///////////////////////////
/////////////////////////// offset Plots
///////////////////////////
module OffsetPlots = 
    correctedIntensities
    |> Array.mapi (fun i lane -> 
        lane
        |> Array.filter (fun x -> Normalization.intersUnique.Contains x.ID) 
        |> Array.mapi (fun j protein -> 
            i,protein.ID,protein.LFQ |> Array.indexed |> Array.maxBy snd |> fst
            )
        )
    |> Array.concat
    |> Array.groupBy (fun (lane,id,maxIntSliceIndex) -> id)
    |> Array.map (fun (id,items) -> 
        let medianSlice = items |> Seq.map trd3 |> Seq.median
        items 
        |> Array.map (fun (lane,_,maxIntSliceIndex) -> 
            lane,maxIntSliceIndex,maxIntSliceIndex-medianSlice
            )
        )
    |> Array.concat
    |> Array.groupBy fst3
    |> Array.map (fun (lane,items) -> 
        items 
        |> Array.map (fun (_,b,c) -> b,c)
        |> Array.groupBy fst
        |> Array.sortBy fst
        |> Array.map (fun (slice,it) -> 
            let offset  = it|> Array.map snd
            let xAx = "sl " + string slice
            Chart.BoxPlot(y=offset,Name=xAx,Color=colors.[lane])
            )
        |> Chart.Combine
        )
    |> Chart.Combine
    |> styleChart "sliceIndex" "offset to median (maxLFQSliceIndex - medianSliceIndex)"
    |> Chart.withSize (1000.,900.)
    |> Chart.Show


    correctedIntensities
    |> Array.map (Array.filter (fun x -> Normalization.intersUnique.Contains x.ID) >> Array.sortBy (fun x -> x.ID))
    |> JaggedArray.transpose
    |> Array.filter (fun lanes -> 
        lanes 
        |> Array.filter (fun x -> Seq.median x.LFQ = 0.)
        |> Array.length
        |> fun reallySignalCount -> reallySignalCount > 4
        )
    |> Array.concat
    //1.
    |> Array.groupBy (fun x -> x.ID)
    |> Array.filter (fun (_,it) -> it.Length = 6)
    |> Array.map (fun (id,items) -> 
        //get median trace
        let mediantrace = 
            items 
            |> Array.map (fun x -> x.LFQ)
            |> JaggedArray.transpose
            |> Array.map Seq.mean //here median?
        
        let indexMaxLFQMean =
            mediantrace
            |> Array.indexed
            |> Array.maxBy snd
            |> fst

        let indicesOfMaxLFQ = 
            items
            |> Array.map (fun x -> x.LFQ |> Array.indexed |> Array.maxBy snd |> fst)

        indicesOfMaxLFQ
        |> Array.mapi (fun laneIndex single -> 
            let offset = single - indexMaxLFQMean
            {|LaneID=laneIndex;SliceID=single;SliceOffset=offset|}
            )
        )
    |> Array.concat
    |> Array.groupBy (fun x -> x.LaneID)
    |> Array.sortBy fst
    |> Array.map (fun (laneID,items) -> 
        items 
        |> Array.groupBy (fun t -> t.SliceID)
        |> Array.sortBy fst
        |> Array.map (fun (sliceID,items) -> 
            let offsets = items |> Array.map (fun x -> x.SliceOffset)
            printfn "%i\t%i\t%i\t" laneID sliceID items.Length
            Chart.BoxPlot(y=offsets,Name="sI " + string sliceID,Color=colors.[laneID])//|> Chart.withTraceName (sprintf "%i" items.Length)
            )
        )
    |> Array.concat
    |> Chart.Combine
    |> styleChart "sliceIndex" "offset to mean (maxLFQSliceIndex - medianSliceIndex)"
    |> Chart.withSize (1000.,900.)
    |> Chart.Show




    //////////////////////////////////
    ////////////////////////////////// plot peak coverage
    //////////////////////////////////
    //////////////////////////////////

    correctedIntensities
    |> Array.mapi (fun i lane -> 
        lane
        |> Array.map (fun protein -> 
            let maxIntens = protein.LFQ |> Array.max
            protein.LFQ |> Array.filter (fun x -> x > 0.2*maxIntens)
            |> Array.length
            |> fun x -> float x 
            )
        |> fun j -> Chart.BoxPlot(y=j,Jitter=0.2,Boxpoints=Boxpoints.All,Boxmean=BoxMean.True,Color=colors.[i])
        )
    |> Chart.Combine
    |> styleChart "" "signal / lane"
    |> Chart.Show


    let getPeakWidths signal percentageForMinIntensity =
        let minSig = signal |> Array.max |> fun x -> x * percentageForMinIntensity
        let rec loop i acc current =
            if i = signal.Length-1 then 
                if signal.[i] > minSig then 
                    if current = 0 then 
                        1::acc 
                    else (current+1)::acc
                else 
                    if current = 0 then acc else (current)::acc
            else 
                if current <> 0 then 
                    if signal.[i] < minSig then 
                        loop (i+1) (current::acc) 0
                    else loop (i+1) acc (current+1)
                else 
                    if signal.[i] > minSig then
                        loop (i+1) acc 1
                    else loop (i+1) acc 0
        loop 0 [] 0

    let plotPeaksPerLane percentageForMinIntensity = 
        correctedIntensities
        |> Array.mapi (fun i lane -> 
            lane
            |> Array.map (fun protein -> 
                getPeakWidths protein.LFQ percentageForMinIntensity
                |> Seq.length
                )
            |> fun j -> Chart.BoxPlot(y=j,Name=laneHeader.[i],Jitter=0.2,Boxpoints=Boxpoints.All,Boxmean=BoxMean.True,Color=colors.[i])
            )
        |> Chart.Combine
        |> styleChart "" "peaks / lane"
        |> Chart.withTitle (string (int (10. * percentageForMinIntensity)) + "% threshold" )

    let plotPeaksPerLaneHisto percentageForMinIntensity = 
        correctedIntensities
        |> Array.mapi (fun i lane -> 
            lane
            |> Array.map (fun protein -> 
                getPeakWidths protein.LFQ percentageForMinIntensity
                |> Seq.length
                )
            )
        |> Array.concat        
        |> fun x -> Chart.Histogram(x,HistNorm=HistNorm.Probability,Color="grey")
        |> styleChart "" "peaks / lane"
        |> Chart.withTitle (string (int (10. * percentageForMinIntensity)) + "% threshold" )



    let plotPeaksWidth percentageForMinIntensity = 
        correctedIntensities
        |> Array.mapi (fun i lane -> 
            lane
            |> Array.map (fun protein -> 
                getPeakWidths protein.LFQ percentageForMinIntensity
                )
            |> Seq.concat
            |> fun j -> Chart.BoxPlot(y=j,Name=laneHeader.[i],Jitter=0.2,Boxpoints=Boxpoints.All,Boxmean=BoxMean.True,Color=colors.[i])
            )
        |> Chart.Combine
        |> styleChart "" "peakwidth / lane"
        |> Chart.withTitle (string (int (100. * percentageForMinIntensity)) + "% threshold" )

    let plotPeaksWidthHisto percentageForMinIntensity = 
        correctedIntensities
        |> Array.mapi (fun i lane -> 
            lane
            |> Array.map (fun protein -> 
                getPeakWidths protein.LFQ percentageForMinIntensity
                )
            |> Seq.concat
            )
        |> Seq.concat        
        |> fun x -> Chart.Histogram(x,HistNorm=HistNorm.Probability,Color="grey")
        |> styleChart "" "peakwidth / lane"
        |> Chart.withTitle (string (int (100. * percentageForMinIntensity)) + "% threshold" )

    [
        plotPeaksPerLane 0.05
        plotPeaksPerLane 0.1
        plotPeaksPerLane 0.2
        plotPeaksWidth   0.05
        plotPeaksWidth   0.1
        plotPeaksWidth   0.2
    ]
    |> Chart.Stack 3
    |> Chart.withSize (1500.,1000.)
    |> Chart.withTitle ("5% (left), 10% (middle) and 20% (right) threshold" )
    |> Chart.Show

    [
        plotPeaksPerLaneHisto 0.05
        plotPeaksPerLaneHisto 0.1
        plotPeaksPerLaneHisto 0.2
        plotPeaksWidthHisto   0.05
        plotPeaksWidthHisto   0.1
        plotPeaksWidthHisto   0.2
    ]
    |> Chart.Stack 3
    |> Chart.withSize (1500.,1000.)
    |> Chart.withTitle ("5% (left), 10% (middle) and 20% (right) threshold" )
    |> Chart.Show






























//////////////////////////////////
////////////////////////////////// distances
//////////////////////////////////
//////////////////////////////////




let getDTWeuc (x:float []) (y:float []) = 
    let dtw = DTW.distanceDefault x y 
    let euc = ML.DistanceMetrics.euclidean x y
    let meanInt = max (Seq.mean x) (Seq.mean y)
    //(dtw + euc) / meanInt

    euc / meanInt




let getWarpDistancesOfSignals (input:float [] []) =
    let inputCount = input.Length
    let normedInput = 
        //input |> Array.map DTW.zNorm
        //input |> Array.map (fun x -> x |> Array.map (fun sx -> sx / (Seq.average x)))
        input
    let rec loop i acc =
        if i >= inputCount-1 then 
            acc
        else
            normedInput.[(i+1)..]
            |> Array.map (fun x -> 
                //let dtwDist = DTW.distanceDefault x normedInput.[i]
                //let euc = ML.DistanceMetrics.euclidean x normedInput.[i]
                //dtwDist + euc
                //log(dtwDist)
                //log(dtwDist + euc)
                getDTWeuc x normedInput.[i]
                )
            |> fun distances -> 
                loop (i+1) (distances::acc)
    loop 0 []
    |> Array.concat




//////////////////////////////////
////////////////////////////////// Writer (combined Funktion)
//////////////////////////////////
//////////////////////////////////

let combinedFunction intMeasurement= 
    let getPeaks signal (percentageForMinIntensity :float) =
        let minSig = signal |> Array.max |> fun x -> x * percentageForMinIntensity
        let sigLength = signal.Length
        [for i=1 to signal.Length - 2 do 
                let currSignal = signal.[i]
                if currSignal > signal.[i-1] && currSignal > signal.[i+1] && currSignal > minSig then yield (i+1)]
        |> fun acc -> if signal.[0] > signal.[1] && signal.[0] > minSig then 1::acc else acc
        |> fun acc -> if signal.[sigLength-1] > signal.[sigLength-2] && signal.[sigLength-1]> minSig then sigLength::acc else acc

    let dataset = 
        match intMeasurement with
        | IntMeasurement.LFQ        -> correctedLFQs            |> Array.map (fun lane -> lane |> Array.filter (fun x -> not (x.ID.Contains ";") && not (x.ID.Contains "iRT")))
        | IntMeasurement.IBAQ       -> correctedIBAQss          |> Array.map (fun lane -> lane |> Array.filter (fun x -> not (x.ID.Contains ";") && not (x.ID.Contains "iRT")))
        | IntMeasurement.Intensity  -> correctedIntensities     |> Array.map (fun lane -> lane |> Array.filter (fun x -> not (x.ID.Contains ";") && not (x.ID.Contains "iRT")))

    dataset
    |> Array.mapi (fun laneIndex lane -> 
        lane
        //|> Array.filter (fun protein -> Normalization.intersUniqueMin2.Contains protein.ID)
        //|> Array.filter (fun protein -> Normalization.intersUnique.Contains protein.ID)
        |> Array.map (fun x -> 
            let intensities = 
                match intMeasurement with
                | IntMeasurement.LFQ            -> x.LFQ
                | IntMeasurement.IBAQ           -> x.IBaq
                | IntMeasurement.Intensity      -> x.Intensities
            laneIndex,x.ID,intensities)
        )
    |> Array.concat
    |> Array.groupBy snd3
    //|> Array.sortByDescending (fun (id,it) -> it|> Array.map snd|> JaggedArray.transpose |> Array.map Seq.mean |> Seq.indexed |> Seq.maxBy snd |> fst)
    |> Array.map (fun (id,x) -> 
        let id' = (id.Split '.').[0..1] |> String.concat "."
        let (wt,wtCount)  = 
            let empty :float [] []= Array.init 3 (fun _ -> Array.zeroCreate 36)
            let intensities = x |> Array.filter (fun (lI,id,lfq) -> lI <=2)  
            for elem in intensities do empty.[fst3 elem] <- trd3 elem
            //for i=0 to intensities.Length-1 do empty.[i] <- intensities.[i]
            empty,(intensities |> Array.map trd3 |> Array.filter (fun x -> (Array.exists (fun t -> t>0.) x))).Length
        let wtMeans = wt |> JaggedArray.transpose |> Array.map Seq.mean
        let (mut,mutCount)  =
            let empty :float [] []= Array.init 3 (fun _ -> Array.zeroCreate 36)
            let intensities = x |> Array.filter (fun (lI,id,lfq) -> lI >=3)
            for elem in intensities do empty.[fst3 elem - 3] <- trd3 elem
            //for i=0 to intensities.Length-1 do empty.[i] <- intensities.[i]
            empty,(intensities |> Array.map trd3 |> Array.filter (fun x -> (Array.exists (fun t -> t>0.) x))).Length
        let mutMeans = mut |> JaggedArray.transpose |> Array.map Seq.mean
        let ttestSums = 
            let wtSums = (wt |> Array.map Seq.sum |> vector)
            let mutSums = (mut |> Array.map Seq.sum |> vector)
            //printfn "%A %A" wtSums mutSums
            if Seq.mean wtSums = 0. then
                if Seq.mean mutSums = 0. then nan
                else (Testing.TTest.twoSample false mutSums wtSums).PValue
            else (Testing.TTest.twoSample false wtSums mutSums).PValue
                
        let dH0_wt  = getWarpDistancesOfSignals wt
        let dH0_mut = getWarpDistancesOfSignals mut
        let combinedH0Distances = Array.append dH0_wt dH0_mut
        [id' + "   wt";id' + " mu"],[wtMeans;mutMeans;(*x.[0..2] |> Array.map snd |> JaggedArray.transpose |> Array.map Seq.mean*)],(combinedH0Distances,ttestSums,(wtCount,mutCount))
        )
    |> Array.unzip3
    |> fun (rowNames,heatmapdata,h0andTests) -> 
        //log of distance + euclidean
        let (h0Distances,pVal,replicates) = 
            Array.unzip3 h0andTests

        let (listString,distances) =
            heatmapdata
            |> Array.mapi  (fun i x -> 
                let d = getDTWeuc x.[0] x.[1]
                let string = sprintf "%s\t%f" ((rowNames.[i].[0].Split ' ').[0]) d 
                string,d)
            |> Array.unzip
        
        let sortedIndices = 
            distances 
            |> Array.indexed
            |> Array.sortBy snd
            |> Array.map fst
            
        let sortedRownames =
            sortedIndices 
            |> Array.map (fun i -> rowNames.[i])

        let pValSorted = 
            sortedIndices 
            |> Array.map (fun i -> pVal.[i])

        let replicateSorted = 
            sortedIndices 
            |> Array.map (fun i -> replicates.[i])


        let distanceHeatmap =
            distances
            |> Array.sort
            |> Array.map (fun x -> [|[|x|];[|x|]|])
            |> Seq.concat
            |> fun x -> 
                Chart.Heatmap(x,RowNames=(sortedRownames|> Seq.concat))
                |> Chart.withX_Axis(LinearAxis.init(Showticklabels=false,Tickfont=Font.init(Color="white"),Ticks=TickOptions.Empty))

    
        let traces = 
            sortedIndices
            |> Array.map (fun i -> heatmapdata.[i])
            |> Array.map (fun x -> [| DTW.zNorm x.[0];DTW.zNorm x.[1]|])
            //DTW.zNorm [|0.;0.;0.;|]

        let peaks =
            sortedIndices
            |> Array.map (fun i -> heatmapdata.[i])
            |> Array.map (fun x -> 
                "[" + (getPeaks x.[0] 0.1 |> List.map string |> String.concat ";") + ";]",
                "[" + (getPeaks x.[1] 0.1 |> List.map string |> String.concat ";") + ";]"
                )

        //let rplicateCount = 
        //    sortedIndices
        //    |> Array.map (fun i -> heatmapdata.[i])
        //    |> Array.map (fun x -> x.[0] x.[1]|])

        let tracesOrig = 
            sortedIndices
            |> Array.map (fun i -> heatmapdata.[i])
            |> Array.map (fun x -> [|x.[0];x.[1]|])

        let traceHeatmap = 
            Chart.Heatmap((traces|> Array.concat),ColNames=[1..36],RowNames=(sortedRownames|> Seq.concat),Showlegend=false,Colorbar=false)
            |> Chart.withColorBarStyle("int",Side.Left)
            |> Chart.withX_AxisStyle "Slice index"

        let h0Distances = h0Distances |> Array.concat
        let h0Size = h0Distances.Length
        let cutOffIndex =0.95 * (float h0Size)
        let cutOffThreshold = (h0Distances |> Array.sort).[int cutOffIndex]
    
        //distances
        //|> Array.sort
        //|> Array.iteri (fun i difference -> 
        //    if difference > cutOffThreshold then 
        //        printfn "%s\t%f\t%s\t%s\t%s" sortedRownames.[i].[0] difference (Annotation.jgi55ToSynStr sortedRownames.[i].[0]) (traces.[i].[0] |> Array.map string |> String.concat "\t") (traces.[i].[1]|> Array.map string |> String.concat "\t")
        //    )
        
        distances
        |> Array.sort
        |> Array.mapi (fun i difference -> 
            if i%10=0 then printfn "%i" i
            let name = (sortedRownames.[i].[0].Split ' ').[0]
            let synonym = "test"//(Annotation.jgi55ToSynStr name)
            let wtmeans  = tracesOrig.[i].[0]
            let mutmeans = tracesOrig.[i].[1] 
            let wtMeanSum = Seq.sum wtmeans
            let mutMeanSum = Seq.sum mutmeans
            let wtMeansStr= wtmeans |> Array.map string |> String.concat "\t"
            let mutMeansStr= mutmeans |> Array.map string |> String.concat "\t"
            let pValSums = pValSorted.[i]
            let dtweucAdj = getDTWeuc wtmeans mutmeans
            let dtw = DTW.distanceDefault wtmeans mutmeans
            let euc = ML.DistanceMetrics.euclidean wtmeans mutmeans
            let maxMean = max (Seq.mean wtmeans) (Seq.mean mutmeans)
            //printfn "%s\t%f\t%s\t%s\t%s" sortedRownames.[i].[0] difference (Annotation.jgi55ToSynStr sortedRownames.[i].[0]) (traces.[i].[0] |> Array.map string |> String.concat "\t") 
            let (mapMan,mapManDes) = 
                "test","test"//(BioFSharp.BioDB.FaToolDb.Queries.getOntologyterms BioDB.FaToolDb.MapManOntology name).OntologyGroups |> List.unzip |> fun (a,b) -> (a |> String.concat ";").Replace("GMM:",""),b |> String.concat ";" 
            let annotationGO = "test"//(BioFSharp.BioDB.FaToolDb.Queries.getOntologyterms BioDB.FaToolDb.GeneOntology name).OntologyGroups |> List.map snd |> String.concat ";" 
            let annotationLoc = "test"//(BioFSharp.BioDB.FaToolDb.Queries.getOntologyterms BioDB.FaToolDb.SubCellularLocalisationOntology name).OntologyGroups |> List.map snd |> String.concat ";" 
            //sprintf $"{name}\t{synonym}\t{annotation}\t{pValSums}\t{difference}\t{dtw}\t{euc}\t{maxMean}\t{wtMeansStr}\t{mutMeansStr}"
            
            //let linkToFile = "=HYPERLINK(LEFT(CELL(\"filename\", $A$1), FIND(\"[\",CELL(\"filename\",$A$1))-1) & \"plots\LFQ\\" + synonym + "_" + name + ".html\",\"" + name + "\")"
            let linkToFile = 
                match intMeasurement with
                | IntMeasurement.LFQ -> "=HYPERLINK(LEFT(CELL(\"filename\", $A$1), FIND(\"[\",CELL(\"filename\",$A$1))-1) & \"plots\LFQ\\" + synonym + "_" + name + ".html\",\"" + name + "\")"
                | IntMeasurement.Intensity -> "=HYPERLINK(LEFT(CELL(\"filename\", $A$1), FIND(\"[\",CELL(\"filename\",$A$1))-1) & \"plots\\" + synonym + "_" + name + ".html\",\"" + name + "\")"
            sprintf $"{linkToFile}\t{synonym}\t{mapManDes}\t{mapMan}\t{annotationGO}\t{annotationLoc}\t{fst replicateSorted.[i]}\t{snd replicateSorted.[i]}\t{pValSums}\t{difference}\t{dtw}\t{euc}\t{maxMean}\t{fst peaks.[i]}\t{snd peaks.[i]}\t{wtMeansStr}\t{mutMeansStr}"
            )
        |> fun x -> 
            printfn "Name\tSynonym\tMapMan\tMapManTerm\tGO\tSubLoc\treplicatesWT\treplicatesMut\tpValSum\tdiff_adj\tdtw\teuc\tmaxMean\tpeaksWT\tpeaksMut\twtMean1\twtMean2\twtMean3\twtMean4\twtMean5\twtMean6\twtMean7\twtMean8\twtMean9\twtMean10\twtMean11\twtMean12\twtMean13\twtMean14\twtMean15\twtMean16\twtMean17\twtMean18\twtMean19\twtMean20\twtMean21\twtMean22\twtMean23\twtMean24\twtMean25\twtMean26\twtMean27\twtMean28\twtMean29\twtMean30\twtMean31\twtMean32\twtMean33\twtMean34\twtMean35\twtMean36\tmutMean1\tmutMean2\tmutMean3\tmutMean4\tmutMean5\tmutMean6\tmutMean7\tmutMean8\tmutMean9\tmutMean10\tmutMean11\tmutMean12\tmutMean13\tmutMean14\tmutMean15\tmutMean16\tmutMean17\tmutMean18\tmutMean19\tmutMean20\tmutMean21\tmutMean22\tmutMean23\tmutMean24\tmutMean25\tmutMean26\tmutMean27\tmutMean28\tmutMean29\tmutMean30\tmutMean31\tmutMean32\tmutMean33\tmutMean34\tmutMean35\tmutMean36"
            
            x |> Array.iter (printfn "%s")
        
        let shape() = Shape.init(ShapeType.Line,cutOffThreshold,cutOffThreshold,0.,0.05)
        [
        h0Distances
        |> fun x -> Chart.Histogram(x,HistNorm=HistNorm.Probability)
        |> Chart.withShape (shape())
        |> Chart.withTraceName "h0_distances"
        |> styleChartwR "" "count" (0.,100.)
        distances
        |> fun x -> Chart.Histogram(x,HistNorm=HistNorm.Probability)
        //|> Chart.withShape (shape())
        |> Chart.withTraceName "vs distances"
        |> styleChartwR "DWTeuc_adj" "count" (0.,100.)
        ]
        |> Chart.Stack 1
        |> Chart.Show

        [[
        traceHeatmap
        distanceHeatmap
        ]]
        |> fun c -> Chart.Grid(c,true)
        |> Chart.withSize (900.,900.)
        |> Chart.withMarginSize (150.)
        |> Chart.withTitle "Migration distances of WT vs Mut"
        |> Chart.ShowWithDescription {Heading ="Interactive chart of migration distances";Text="(left) zNormed intensities of wild type and mutant.<br>(right) Adjusted euclidean distance of each protein pair.<br><br>Note: Distance is 36 if signal of WT or Mut is missing. Click and zoom vertically for more details"}

        listString

combinedFunction IntMeasurement.Intensity





//////////////////////////////////
////////////////////////////////// plot signals
//////////////////////////////////
//////////////////////////////////
let plotSignals save weight cre intmeasure =
    
    let synonym = Annotation.jgi55ToSynStr cre
    let original = 
        datasets
        |> Array.map (Array.filter (fun x -> (x.ID |> String.filter (fun s -> s = 'C') |> fun c -> c.Length = 1))) //new
        |> Array.mapi (fun laneIndex x -> 
            x 
            |> Array.map (fun x -> x)
            |> Array.filter (fun (x) -> x.ID.StartsWith cre)
            |> fun x -> 
                if x = [||] then 
                    laneIndex,Array.init 36 (fun _ -> 0.)
                else 
                    if x.Length > 2 then printfn "WARNING: %A" x
                    match intmeasure with
                    | IntMeasurement.LFQ        -> laneIndex,x.[0].LFQ
                    | IntMeasurement.Intensity  -> laneIndex,x.[0].Intensities
                    | IntMeasurement.IBAQ       -> laneIndex,x.[0].IBaq
            )

    let originalChart = 
        original
        |> Array.map (fun (j,x) -> x |> Seq.mapi (fun i x-> i+1,x) |> fun x -> Chart.Line(x,Name=laneHeader.[j] + " raw",Color=colors.[j]))
        |> Chart.Combine
    
    let normed = 
        let normedData = 
            match intmeasure with
            | IntMeasurement.LFQ       -> correctedLFQs
            | IntMeasurement.Intensity -> correctedIntensities
            | IntMeasurement.IBAQ      -> correctedIBAQss
            
        normedData
        |> Array.map (Array.filter (fun x -> (x.ID |> String.filter (fun s -> s = 'C') |> fun c -> c.Length = 1))) //new
        |> Array.mapi (fun laneIndex x -> 
            x 
            |> Array.map (fun x -> x)
            |> Array.filter (fun (x) -> x.ID.StartsWith cre)
            |> fun x -> 
                if x = [||] then 
                    laneIndex,Array.init 36 (fun _ -> 0.)
                else 
                    if x.Length > 2 then printfn "WARNING: %A" x
                    match intmeasure with
                    | IntMeasurement.LFQ        -> laneIndex,x.[0].LFQ
                    | IntMeasurement.Intensity  -> laneIndex,x.[0].Intensities
                    | IntMeasurement.IBAQ       -> laneIndex,x.[0].IBaq
            )
        //|> Array.concat
        //|> Array.mapi (fun j (laneIndex,x) -> laneIndex,x.LFQ)
        
    let normedChart = 
        normed
        |> Array.map (fun (j,x) -> x |> Seq.mapi (fun i x-> i+1,x) |> fun x -> Chart.Line(x,Name=laneHeader.[j] + " norm",Color=colors.[j]) )
        |> Chart.Combine

    let wtTraces  = (normed |> Array.filter (fun (i,x) -> i <= 2) |> Array.map snd)
    let mutTraces = (normed |> Array.filter (fun (i,x) -> i >= 3) |> Array.map snd)
    //let h0Distances = 
    //    Array.append 
    //        (getWarpDistancesOfSignals wtTraces)
    //        (getWarpDistancesOfSignals mutTraces )
    //    |> Array.map (sprintf "%.2f") |> String.concat ", "

    let wt = wtTraces |> JaggedArray.transpose |> Array.map Seq.mean
    let mu = mutTraces |> JaggedArray.transpose |> Array.map Seq.mean
    let wtStd = wtTraces |> JaggedArray.transpose |> Array.map Seq.stDev
    let muStd = mutTraces |> JaggedArray.transpose |> Array.map Seq.stDev

    let meanChart = 
        [
        Chart.Line(wt |> Seq.mapi (fun i x-> i+1,x),Color=colors.[0],Name="wt mean")|> Chart.withYErrorStyle(wtStd)
        Chart.Line(mu |> Seq.mapi (fun i x-> i+1,x),Color=colors.[5],Name="mut mean")|> Chart.withYErrorStyle(muStd)
        ]
        |> Chart.Combine
        //|> Chart.withAxisAnchor(Y=1)

    //let dTW = DTW.distanceDefault wt mu
    let euc = ML.DistanceMetrics.euclidean wt mu
    let dfi = getDTWeuc wt mu

    ///Testing
    let wtSums = wtTraces |> Array.map Seq.sum  |> vector
    let mutSums = mutTraces |> Array.map Seq.sum |> vector
    let sumFoldChange = Seq.map2 (fun w m -> m/w) wtSums mutSums |> Seq.mean

    let ttest = 
        if Seq.mean wtSums = 0. then
            if Seq.mean mutSums = 0. then nan
            else (Testing.TTest.twoSample false mutSums wtSums).PValue
        else (Testing.TTest.twoSample false wtSums mutSums).PValue
    
    let ttestPval = ttest
    //contasts
    let wtSlices  = (wtTraces |> JaggedArray.transpose)
    let mutSlices = (mutTraces |> JaggedArray.transpose)
    let appendedSlices = Array.append wtSlices mutSlices
    //let contrasts = 
    //    Array.init 36 (fun testIndex -> 
    //        let tmpContrast :float [] = Array.zeroCreate (36*2)
    //        tmpContrast.[testIndex] <- 1.
    //        tmpContrast.[testIndex+36] <- 1.
    //        tmpContrast
    //        )
    //let contrastResults = 
    //    Testing.PostHoc.fishersLSD contrasts appendedSlices //auf gar keinen fall einen post hoc test nehmen!! wegen variance estimation!
    let contrastResults = 
        Array.mapi2 (fun i w m -> 
            if Seq.mean w = 0. (*|| Seq.mean m = 0.*) then 
                if Seq.mean m <> 0. then 
                    i+1,(Testing.TTest.twoSample false (vector m) (vector w)).PValue
                else i+1,nan
            else 
                i+1,(Testing.TTest.twoSample false (vector w) (vector m)).PValue
            ) 
            wtSlices 
            mutSlices
    //let contrastResults = Testing.PostHoc.fishersLSD contrasts appendedSlices //auf gar keinen fall einen post hoc test nehmen!! wegen variance estimation!
    let contrastResultStrings = 
        contrastResults 
        |> Array.map (fun (i,x) -> $"{i}\t\t%.2f{x}")
        |> Array.chunkBySize 9
        |> fun chunks -> 
            chunks.[0]
            |> Array.mapi (fun j chunk9 -> 
                sprintf "%s\t\t%s\t\t%s\t\t%s" chunk9 chunks.[1].[j] chunks.[2].[j] chunks.[3].[j]
                )
        |> String.concat "<br>"
    let pValues =
        contrastResults 
        |> Array.partition (fun (i,x) -> x < 0.05)
        |> fun (pos,neg) -> 
            [Chart.Column(pos|> Array.map (fun (i,x) -> i,- log10 x),Color="green");Chart.Column(neg|> Array.map (fun (i,x) -> i,- log10 x),Color="grey")] 
            |> Chart.Combine
            |> Chart.withTraceName "pVals Welch"
    
    let intMeasureStr =
        match intmeasure with
        | IntMeasurement.LFQ        -> "LFQ"
        | IntMeasurement.Intensity  -> "Intensities"
        | IntMeasurement.IBAQ       -> "IBaq"

    let annotation = 
        Annotation.jgi55ToDesStr ((cre.Split '.').[0..1] |> String.concat ".")

    [
    //[pValues       |> styleChartwR "" "-log10 pValue"(1.,36.)]
    [meanChart     |> styleChartwR "" "means"        (1.,36.)]
    [normedChart   |> styleChartwR "" (intMeasureStr + " normed")   (1.,36.)]
    [originalChart |> styleChartwR "" (intMeasureStr + " raw")      (1.,36.)]
    ]
    |> fun t -> Chart.Grid(t,true,rowOrder=LayoutGridRowOrder.BottomToTop)
    |> Chart.withTitle (cre + " " + synonym)
    |> Chart.withSize (1100.,800.)
    |> fun t -> 
        if not save then 
            t
            //|> Chart.ShowWithDescription {Heading="";Text= $"dwt_distance: %.2f{dTW}<br>euclidean: %.2f{euc}<br>meanInt: %.2f{max (Seq.mean wt) (Seq.mean mu)}<br>myDist: %.2f{dfi}<br><br>MapMan: {annotation}<br><br>H0_distances: {h0Distances}<br><br>TTest LFQSums: %.4f{ttestPval}<br><br>TestSingleContrasts:<br>{contrastResultStrings}"}
            |> Chart.ShowWithDescription {Heading="";Text= $"Trivial name: {synonym}<br><br>Euclidean distance: %.2f{euc}<br>Mean Intensity (WT): %.2f{Seq.mean wt}<br>Mean Intensity (Mut): %.2f{Seq.mean mu}<br>Distance: %.2f{dfi}<br><br>MapMan: {annotation}<br><br>p value of intensity sums Welch test: %.4f{ttestPval}"}
        else 
            let path = 
                if weight then (@"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\results\plots\distance\" + sprintf "%03.0f_%s_%s" dfi synonym cre + ".html" ) 
                else (@"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\results\plots\" + sprintf "%s_%s" synonym cre + ".html" ) 
            t 
            |> Chart.SaveHtmlWithDescriptionAs 
                path
                //{Heading="";Text= $"dwt_distance: %.2f{dTW}<br>euclidean: %.2f{euc}<br>meanInt: %.2f{max (Seq.mean wt) (Seq.mean mu)}<br>myDist: %.2f{dfi}<br><br>H0_distances: {h0Distances}<br><br>TTest LFQSums: %.4f{ttestPval}<br><br>TestSingleContrasts:<br>{contrastResultStrings}"}
                //{Heading="";Text= $"dwt_distance: %.2f{dTW}<br>euclidean: %.2f{euc}<br>meanInt: %.2f{max (Seq.mean wt) (Seq.mean mu)}<br>myDist: %.2f{dfi}<br><br>MapMan: {annotation}<br><br>H0_distances: {h0Distances}<br><br>TestSingleContrasts:<br>{contrastResultStrings}"}
                {Heading="";Text= $"Trivial name: {synonym}<br><br>Euclidean distance: %.2f{euc}<br>Mean Intensity (WT): %.2f{Seq.mean wt}<br>Mean Intensity (Mut): %.2f{Seq.mean mu}<br>Distance: %.2f{dfi}<br><br>MapMan: {annotation}<br><br>p value of intensity sums Welch test: %.4f{ttestPval}"}


    //printfn "diff_adj %f" (getDTWeuc wt mu)
    //printfn "euc %f" (ML.DistanceMetrics.euclidean wt mu)
    //printfn "dtw %f" (DTW.distanceDefault wt mu)
    

plotSignals false false "Cre12.g493250" IntMeasurement.Intensity


datasets
|> Array.map (fun x -> 
    x 
    |> Array.map (fun protein -> 
        protein.ID
        )
    )
|> Array.concat
|> Array.distinct
|> Array.filter (fun x -> not (x.Contains ";"))
|> Array.mapi (fun i x ->
    printfn "%i" i
    plotSignals true false ((x.Split '.').[0..1] |> String.concat ".") IntMeasurement.Intensity
    )




//check cases with unique (PSHB4...)
datasets
|> Array.map (fun x -> 
    x 
    |> Array.map (fun protein -> 
        protein.ID
        )
    )
|> Array.concat
|> Array.distinct
|> Array.filter (fun x -> not (x.Contains ";"))
|> Array.mapi (fun i cre ->
    let vorher = 
        datasets 
        |> Array.mapi (fun laneIndex x -> 
            x 
            |> Array.map (fun x -> x)
            |> Array.filter (fun (x) -> x.ID.StartsWith cre)
        )
        |> Array.mapi (fun i x -> i,x)


    let nachher = 
        datasets |> Array.map (Array.filter (fun x -> (x.ID |> String.filter (fun s -> s = 'C') |> fun c -> c.Length = 1))) //new
        |> Array.mapi (fun laneIndex x -> 
            x 
            |> Array.map (fun x -> x)
            |> Array.filter (fun (x) -> x.ID.StartsWith cre)
            )
        |> Array.mapi (fun i x -> i,x)
    (cre,vorher=nachher),(vorher |> Array.map (fun (i,x) -> sprintf "%i" x.Length),nachher |> Array.map (fun (i,x) -> sprintf "%i" x.Length))
    )
|> Array.filter (fun (a,b) -> not (snd a))
|> Array.iteri (fun i (a,b) -> printfn "%i\t%s\t%s\t%s\t%s" i (Annotation.jgi55ToSynStr (fst a)) (fst a) ((fst b) |> String.concat ";") ((snd b) |> String.concat ";"))

    