


#r "nuget: Plotly.NET,2.0.0-alpha2"
#r "nuget: BioFSharp,2.0.0-beta4"
#r "nuget: BioFSharp.BioDB,2.0.0-beta4"
#r "nuget: FSharp.Stats,0.3.0-beta"
#r "nuget: FSharpAux,1.0.0"
#r "netstandard.dll"

open System
open Plotly.NET
open Plotly.NET.Axis
open BioFSharp
open FSharp.Stats
open StyleParam
open FSharpAux
open Fitting.LinearRegression.OrdinaryLeastSquares

#time

FSharp.Stats.ServiceLocator.setEnvironmentPathVariable @"C:\Users\bvenn\source\repos\FSharp.Stats\lib"
FSharp.Stats.Algebra.LinearAlgebra.Service()

let myAxis name = LinearAxis.init(Title=name,Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true)
let myAxisWODes name = LinearAxis.init(Title=name,Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true,Tickfont=Font.init(StyleParam.FontFamily.Arial,Size=6.))
let myAxisRange name range = LinearAxis.init(Title=name,Range=StyleParam.Range.MinMax(range), Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true)
let myAxisRangeWODes name range = LinearAxis.init(Title=name,Range=StyleParam.Range.MinMax(range), Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true,Tickfont=Font.init(StyleParam.FontFamily.Arial,Size=6.))
let myAxisLog name = LinearAxis.init(AxisType=AxisType.Log,Title=name, Mirror=StyleParam.Mirror.All,Ticks=StyleParam.TickOptions.Inside,Showgrid=false,Showline=true)
let styleChart x y chart = chart |> Chart.withX_Axis (myAxis x) |> Chart.withY_Axis (myAxis y)
let styleChartwR x y (min,max) chart = chart |> Chart.withX_Axis (myAxisRange x (min,max)) |> Chart.withY_Axis (myAxis y)


let getReg (vec:Vector<float>) rep = vec.[0..rep]
Vector.init 2 (fun _ -> 2.)

 
type Protein = {
    ID      : string
    MajorID : string
    MolWeight: float
    //Qvalue : float
    //Score : float
    Intensities: float []
    IBaq : float []
    LFQ : float []
    Coverage : float
    SequenceLength : int
    UniquePeptides: int
    } with static member create id majID weight inten ibaq lfq c s u =
            {
            ID = id
            MajorID = majID 
            MolWeight = weight
            Intensities = inten
            IBaq = ibaq
            LFQ = lfq
            Coverage = c
            SequenceLength = s
            UniquePeptides = u
            }

type IntMeasurement = 
    | Intensity
    | LFQ
    | IBAQ

let getIntensity intensityType transform (x:Protein) = 
    match intensityType with
    | IBAQ      -> Array.map transform x.IBaq
    | Intensity -> Array.map transform x.Intensities
    | LFQ       -> Array.map transform x.LFQ

let readFrame foldername =
    System.IO.File.ReadAllLines(@"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\data\Fred\CPLXome LPA2\20180420 MS " + foldername + " txt\proteinGroups.txt")
    |> Array.tail
    |> Array.map (fun x -> 
        //printfn "%s" x
        let tmp = x.Split '\t'
        let id = tmp.[0]
        let majid = tmp.[1]
        let molWeight = tmp.[121]       |> float
        let qValue = tmp.[124]
        let score = tmp.[125]
        let intensity = tmp.[163..198]  |> Array.map float
        let iBAQ = tmp.[200..235]       |> Array.map float
        let LFQ = tmp.[236..271]        |> Array.map float
        let cov = tmp.[118] |> float
        let seqLen = tmp.[122] |> int
        let uniPep = tmp.[4].Split ';' |> Seq.head |> int
        //String.concat "\t" [id;majid;molWeight;qValue;score;String.concat "\t" intensity;String.concat "\t"iBAQ;String.concat "\t"LFQ]
        Protein.create id majid molWeight intensity iBAQ LFQ cov seqLen uniPep
        )
    |> Array.filter (fun x -> 
        not (x.ID.Contains "CON") && not (x.ID.Contains "REV" )
        //&& (x.ID |> String.filter (fun s -> s = 'C') |> fun c -> c.Length = 1)
        )


let wt1  = readFrame "JBbnAida001" 
let wt2  = readFrame "JBbnBida001" 
let wt3  = readFrame "JBbnCida001" 
let mut1 = readFrame "JBbnDida001" 
let mut2 = readFrame "JBbnEida001" 
let mut3 = readFrame "JBbnFida001"  

let datasets = [|wt1;wt2;wt3;mut1;mut2;mut3|]
let colors= [|"F25930";"FABB01";"FFEA48";"48BCD7";"0081C7";"0B3D6F";|]
let laneHeader = [|"wt1";"wt2";"wt3";"mut1";"mut2";"mut3"|]



module Normalization =
    

    /// defines proteins existing in every lane and are unique in protein group
    let intersUnique = 
        [
        wt1  |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        wt2  |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        wt3  |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        mut1 |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        mut2 |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        mut3 |> Array.map (fun x -> x.ID) |> Array.filter (fun x -> (x.Split ';').Length = 1) |> Set.ofArray
        ]
        |> Set.intersectMany


    let plotLaneIntensities (transform:float -> float) name (datasets:Protein[] []) intType = 
        datasets
        |> Array.map (fun lane ->
            lane
            //|> Array.filter (fun protein -> inters.Contains protein.ID)
            |> Array.sumBy (fun t -> Array.sum (Array.map transform (getIntensity intType transform t)))
            )
        |> Array.mapi (fun i x -> 
            //printfn "%f" x
            laneHeader.[i],x)
        //|> Chart.Combine
        |> Chart.Line
        |> styleChart "lane" "intensity sum"
        |> Chart.withTitle name
        |> Chart.Show

    
    //plotLaneIntensities (fun si -> if si = 0. then 0. else log si)  "intersect proteins, log lfq sum"  [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.LFQ 
    plotLaneIntensities id                                          "all proteins,     lfq sum"   [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.LFQ
    plotLaneIntensities id                                          "all proteins,     IBAq sum"  [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.IBAQ
    plotLaneIntensities id                                          "all proteins,     intensity sum"  [|wt1;wt2;wt3;mut1;mut2;mut3|] IntMeasurement.Intensity
    
    let plotSliceIntensities (transform :float -> float) name aggregator (datasets:Protein[] []) intType =
        datasets
        |> Array.map (fun x -> x (*|> Array.filter (fun x -> inters.Contains x.ID)*) |> Array.sortBy (fun x -> x.ID) |> Array.map (fun x -> Array.map transform (getIntensity intType transform x)))
        |> Array.mapi (fun i x -> 
            x 
            |> JaggedArray.transpose
            |> Array.map aggregator
            |> Array.indexed
            |> Chart.Line
            |> Chart.withTraceName laneHeader.[i])
        |> Chart.Combine
        |> styleChart "slice" "intensity sum"
        |> Chart.withTitle name
        |> Chart.withSize (800.,500.)
        |> Chart.Show

    //plotSliceIntensities (fun si -> if si = 0. then 0. else log si)  "All proteins, log intensities sum"   Array.sum [|wt1;wt2;wt3;mut1;mut2;mut3|]
    //plotSliceIntensities id      "All proteins,     intensities sum"   Array.sum [|wt1;wt2;wt3;mut1;mut2;mut3|]

    let correctedValues intType transform = 
        // summed lane intensities
        let summedlanes =
            [|wt1;wt2;wt3;mut1;mut2;mut3|]
            |> Array.map (fun lane ->
                lane
                //|> Array.filter (fun protein -> inters.Contains protein.ID)
                |> Array.sumBy (fun t -> Array.sum (getIntensity intType transform t))
                )
        let laneSumsMean = Seq.mean summedlanes
        let laneSumsCorrection = 
            summedlanes 
            |> Array.map (fun x -> x/laneSumsMean)

        let corrected = 
            [|wt1;wt2;wt3;mut1;mut2;mut3|]
            |> Array.mapi (fun laneIndex lane -> 
                lane 
                |> Array.map (fun protein ->
                    let corrected = 
                        (getIntensity intType transform protein)
                        |> Array.mapi (fun sliceIndex slice -> 
                            slice/laneSumsCorrection.[laneIndex])
                    match intType with
                    | IntMeasurement.LFQ ->         {protein with LFQ = corrected;  Intensities = [||];         IBaq = [||]}
                    | IntMeasurement.Intensity ->   {protein with LFQ = [||];       Intensities = corrected;    IBaq = [||]}
                    | IntMeasurement.IBAQ ->        {protein with LFQ = [||];       Intensities = [||];         IBaq = corrected}
                    )
                )
        corrected
    
//let correctedLFQs =         Normalization.correctedValues IntMeasurement.LFQ            id
//let correctedIBAQss =       Normalization.correctedValues IntMeasurement.IBAQ           id
let correctedIntensities =  Normalization.correctedValues IntMeasurement.Intensity      id

Normalization.plotSliceIntensities id "slice intensity sums (after intensity sum correction)" Array.sum correctedIntensities  IntMeasurement.Intensity
Normalization.plotSliceIntensities id "slice intensity sums" Array.sum datasets  IntMeasurement.Intensity
Normalization.plotLaneIntensities id "slice intensity sums" correctedIntensities  IntMeasurement.Intensity


let mutable setwt1 :Map<string,string>= Map.ofList []
let mutable setwt2 :Map<string,string>= Map.ofList []
let mutable setwt3 :Map<string,string>= Map.ofList []
let mutable setmu1 :Map<string,string>= Map.ofList []
let mutable setmu2 :Map<string,string>= Map.ofList []
let mutable setmu3 :Map<string,string>= Map.ofList []


let writeDataFrame (input:Protein[][]) outputPath =
    input
    |> Array.mapi (fun j sample -> 
        sample
        //|> Array.take 5
        |> Array.filter (fun x -> 
            let ids = 
                x.ID.Split ";" 
                |> Array.map (fun x -> 
                    x.Split([|".t";".p"|],StringSplitOptions.None)
                    |> Seq.head
                )
                |> Array.distinct
                |> Array.length
            not (x.ID.Contains "iRT") &&
            ids = 1 //cedarfilter
            )
        |> Array.mapi (fun i x ->   
            if i%50=0 then printfn "%i_%i" j i
            let id = 
                let tmp = x.ID.Split ';' |> Array.map (fun x -> x.Split([|".t";".p"|],StringSplitOptions.None)) |> Array.map Seq.head |> Array.distinct
                if tmp.Length <> 1 then failwithf "%s" x.ID else Array.head tmp
            let cedarString = 
                sprintf "%s\t%f\t%f\t%i\t%i\t%s" id x.MolWeight x.Coverage x.SequenceLength x.UniquePeptides (x.Intensities |> Array.map string |> String.concat "\t")
            
            sprintf "%s\t%f\t%s" x.ID x.MolWeight (x.Intensities |> Array.map string |> String.concat "\t")
            
            id,cedarString
            )
        )
    |> Array.mapi (fun i sample -> 
        match i with
        | 0 -> setwt1 <- sample |> Map.ofArray
        | 1 -> setwt2 <- sample |> Map.ofArray
        | 2 -> setwt3 <- sample |> Map.ofArray
        | 3 -> setmu1 <- sample |> Map.ofArray
        | 4 -> setmu2 <- sample |> Map.ofArray
        | 5 -> setmu3 <- sample |> Map.ofArray
        let sample = sample |> Array.map snd
        let intensityHeader = Array.init 36 (fun x -> sprintf "Intensity_%02i" (x+1)) |> String.concat "\t"
        //let header = sprintf "Identifier\tTrivialName\tMolWeight\tMapMan\tMapManDescription\tGO\tLocalization\t%s" intensityHeader
        //let header = sprintf "Identifier\tMolWeight\t%s" intensityHeader
        let header = sprintf "CRE identifier (JGI 5.5)\tMolWeight\tCoverage\tprotein length\tunique count\t%s" intensityHeader
        let rows = Array.append [|header|] sample
        ()//System.IO.File.WriteAllLines(outputPath + laneHeader.[i] + ".txt",rows)
        )

//when normed
writeDataFrame correctedIntensities ""//@"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\results\forMichael_v9\intensitiesNormed_cedar"

//when raw
//writeDataFrame datasets "" //@"C:\Users\bvenn\OneDrive - tukl\Projects\Benni\results\forMichael_v9\intensitiesraw_cedar"

let unionSet = 
    Set.unionMany
        [
        setwt1 |> Map.toArray |> Array.map fst |> Set.ofArray
        setwt2 |> Map.toArray |> Array.map fst |> Set.ofArray
        setwt3 |> Map.toArray |> Array.map fst |> Set.ofArray
        setmu1 |> Map.toArray |> Array.map fst |> Set.ofArray
        setmu2 |> Map.toArray |> Array.map fst |> Set.ofArray
        setmu3 |> Map.toArray |> Array.map fst |> Set.ofArray
        ]

let map =
    System.IO.File.ReadAllLines @"C:\Users\bvenn\source\repos\ARCs\AnnotationARC\externals\AnnotationSnapshots\chlamy_jgi55.txt"
    |> Array.skip 2
    |> Array.map (fun x -> 
        let tmp = x.Split '\t'
        tmp.[0],tmp.[6]
        )
    |> Map.ofArray

let globalString = 
    unionSet
    |> Seq.map (fun x -> 
        let id = x
        let getSynonym = if map.ContainsKey id then map.[id] else ""
        let createEmpty = "0\t0.0\t" + (Array.init 36 (fun x -> "0.0") |> String.concat "\t")
        let mutable molWeight = ""
        let mutable seqLenght = "" 
        let modString (str :string) = 
            str.Split '\t'
            |> fun x -> //id x.MolWeight x.Coverage x.SequenceLength x.UniquePep
                let mw = x.[1]
                if molWeight = "" then molWeight <- mw else (if mw <> molWeight then printfn "error mw!! %s %s %s" mw molWeight id)
                let cov = x.[2]
                let seqlength = x.[3]
                if seqLenght = "" then seqLenght <- seqlength else (if seqlength <> seqLenght then printfn "error seqlength!! %s %s %s" seqlength seqLenght id)
                let uniquecount = x.[4]
                [|[|uniquecount|];[|cov|];x.[5..]|]
                [|[|uniquecount|];[|cov|];x.[5..] |> Array.rev|]
            |> Array.concat
            |> String.concat "\t"
        let wt1 = if setwt1.ContainsKey x then modString setwt1.[x] else createEmpty
        let wt2 = if setwt2.ContainsKey x then modString setwt2.[x] else createEmpty
        let wt3 = if setwt3.ContainsKey x then modString setwt3.[x] else createEmpty
        let mu1 = if setmu1.ContainsKey x then modString setmu1.[x] else createEmpty
        let mu2 = if setmu2.ContainsKey x then modString setmu2.[x] else createEmpty
        let mu3 = if setmu3.ContainsKey x then modString setmu3.[x] else createEmpty
        sprintf "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" x getSynonym molWeight seqLenght wt1 wt2 wt3 mu1 mu2 mu3
                )
    |> Array.ofSeq





globalString
|> fun d -> 
    let getGlobalHeader i = 
        let create str = Array.init 38 (fun x -> str) |> String.concat "\t"
        match i with
        | 0 -> create "CC-4533_1"
        | 1 -> create "CC-4533_2"
        | 2 -> create "CC-4533_3"
        | 3 -> create "lpa2_1"
        | 4 -> create "lpa2_2"
        | 5 -> create "lpa2_3"
    let globalHeader = 
        "Identifier\tSynonym\tMolWeight\tSequence length\t" + ([0..5] |> List.map getGlobalHeader |> String.concat "\t")
    let intensityHeader = "peptide count\tcoverage\t" + (Array.init 36 (fun x -> sprintf "slice_%02i" (37 - (x + 1))) |> String.concat "\t")
    let intensityHeader = "peptide count\tcoverage\t" + (Array.init 36 (fun x -> sprintf "slice_%02i" (37 - (x + 1))) |> String.concat "\t")
    
    let header = sprintf "CRE identifier (JGI 5.5)\tSynonym\tMolWeight\tprotein length\t%s\t%s\t%s\t%s\t%s\t%s" intensityHeader intensityHeader intensityHeader intensityHeader intensityHeader intensityHeader
    let rows = Array.append [|globalHeader;header|] d
    //System.IO.File.WriteAllLines(@"C:\Users\bvenn\Desktop\delete\rawCedar2.txt",rows)
    System.IO.File.WriteAllLines(@"C:\Users\bvenn\Desktop\delete\normedCedar2.txt",rows)




correctedIntensities
|> Array.map (fun x -> x |> Array.map (fun x -> x.Intensities))
|> Array.concat
|> Array.concat
|> Array.filter (fun x -> x <> 0.)
|> Array.map log10
|> Chart.Histogram
|> styleChart "log10(intensity)" "count"
|> Chart.Show
